@@ -1,5 +1,5 @@
 /*
-  Copyright(C) 2009-2016 Brazil
+  Copyright(C) 2009-2017 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -22,8 +22,17 @@
 extern "C" {
 #endif
 
+typedef struct _grn_column_cache grn_column_cache;
+
 GRN_API grn_column_flags grn_column_get_flags(grn_ctx *ctx, grn_obj *column);
 
+GRN_API grn_column_cache *grn_column_cache_open(grn_ctx *ctx, grn_obj *column);
+GRN_API void grn_column_cache_close(grn_ctx *ctx, grn_column_cache *cache);
+GRN_API void *grn_column_cache_ref(grn_ctx *ctx,
+                                   grn_column_cache *cache,
+                                   grn_id id,
+                                   size_t *value_size);
+
 #ifdef __cplusplus
 }
 #endif
