@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2009-2012 Brazil
+/*
+  Copyright(C) 2009-2017 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -280,6 +281,7 @@ grn_com_event_init(grn_ctx *ctx, grn_com_event *ev, int max_nevents, int data_si
     ev->msg_handler = NULL;
     memset(&(ev->curr_edge_id), 0, sizeof(grn_com_addr));
     ev->acceptor = NULL;
+    ev->listen_backlog = GRN_COM_EVENT_LISTEN_BACKLOG_DEFAULT;
     ev->opaque = NULL;
 #ifndef USE_SELECT
 # ifdef USE_EPOLL
@@ -338,6 +340,18 @@ grn_com_event_fin(grn_ctx *ctx, grn_com_event *ev)
   return GRN_SUCCESS;
 }
 
+void
+grn_com_event_set_listen_backlog(grn_ctx *ctx, grn_com_event *ev, int backlog)
+{
+  ev->listen_backlog = backlog;
+}
+
+int
+grn_com_event_get_listen_backlog(grn_ctx *ctx, grn_com_event *ev)
+{
+  return ev->listen_backlog;
+}
+
 grn_rc
 grn_com_event_add(grn_ctx *ctx, grn_com_event *ev, grn_sock fd, int events, grn_com **com)
 {
@@ -455,14 +469,12 @@ grn_com_event_del(grn_ctx *ctx, grn_com_event *ev, grn_sock fd)
     } else {
       GRN_LOG(ctx, GRN_LOG_ERROR,
               "%04x| fd(%" GRN_FMT_SOCKET ") not found in ev(%p)",
-              getpid(), fd, ev);
+              grn_getpid(), fd, ev);
       return GRN_INVALID_ARGUMENT;
     }
   }
 }
 
-#define LISTEN_BACKLOG 0x1000
-
 grn_rc
 grn_com_event_start_accept(grn_ctx *ctx, grn_com_event *ev)
 {
@@ -472,7 +484,7 @@ grn_com_event_start_accept(grn_ctx *ctx, grn_com_event *ev)
 
   GRN_API_ENTER;
   if (!grn_com_event_mod(ctx, ev, com->fd, GRN_COM_POLLIN, NULL)) {
-    if (listen(com->fd, LISTEN_BACKLOG) == 0) {
+    if (listen(com->fd, ev->listen_backlog) == 0) {
       com->accepting = GRN_TRUE;
     } else {
       SOERR("listen - start accept");
@@ -1035,7 +1047,7 @@ grn_com_close(grn_ctx *ctx, grn_com *com)
   if (ev) {
     grn_com *acceptor = ev->acceptor;
     grn_com_event_del(ctx, ev, fd);
-    if (acceptor) { grn_com_event_start_accept(ctx, ev); }
+    if (acceptor) { grn_com_event_stop_accept(ctx, ev); }
   }
   if (!com->closed) { grn_com_close_(ctx, com); }
   if (!ev) { GRN_FREE(com); }
@@ -1114,7 +1126,7 @@ grn_com_sopen(grn_ctx *ctx, grn_com_event *ev,
     SOERR("bind");
     goto exit;
   }
-  if (listen(lfd, LISTEN_BACKLOG) < 0) {
+  if (listen(lfd, ev->listen_backlog) < 0) {
     SOERR("listen");
     goto exit;
   }
