@@ -159,6 +159,8 @@ grn_loader_init(grn_loader *loader)
   loader->each = NULL;
   loader->values_size = 0;
   loader->nrecords = 0;
+  loader->n_record_errors = 0;
+  loader->n_column_errors = 0;
   loader->stat = GRN_LOADER_BEGIN;
   loader->columns_status = GRN_LOADER_COLUMNS_UNSET;
   loader->rc = GRN_SUCCESS;
