@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/*
+  Copyright(C) 2011-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -12,18 +13,17 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_DAT_CURSOR_HPP_
-#define GRN_DAT_CURSOR_HPP_
+#pragma once
 
 #include "key.hpp"
 
 namespace grn {
 namespace dat {
 
-class Cursor {
+class GRN_DAT_API Cursor {
  public:
   Cursor() {}
   virtual ~Cursor() {}
@@ -44,5 +44,3 @@ class Cursor {
 
 }  // namespace dat
 }  // namespace grn
-
-#endif  // GRN_DAT_CURSOR_HPP_
