@@ -12,7 +12,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "key-cursor.hpp"
@@ -28,7 +28,7 @@ namespace dat {
 KeyCursor::KeyCursor()
     : trie_(NULL),
       offset_(0),
-      limit_(UINT32_MAX),
+      limit_(MAX_UINT32),
       flags_(KEY_RANGE_CURSOR),
       buf_(),
       count_(0),
@@ -112,8 +112,8 @@ UInt32 KeyCursor::fix_flags(UInt32 flags) const {
 }
 
 void KeyCursor::init(const String &min_str, const String &max_str) {
-  if (offset_ > (UINT32_MAX - limit_)) {
-    max_count_ = UINT32_MAX;
+  if (offset_ > (MAX_UINT32 - limit_)) {
+    max_count_ = MAX_UINT32;
   } else {
     max_count_ = offset_ + limit_;
   }
@@ -133,7 +133,7 @@ void KeyCursor::ascending_init(const String &min_str, const String &max_str) {
   if (max_str.ptr() != NULL) {
     if (max_str.length() != 0) {
       end_buf_ = new UInt8[max_str.length()];
-      std::memcpy(end_buf_, max_str.ptr(), max_str.length());
+      grn_memcpy(end_buf_, max_str.ptr(), max_str.length());
       end_str_.assign(end_buf_, max_str.length());
     }
   }
@@ -206,7 +206,7 @@ void KeyCursor::descending_init(const String &min_str, const String &max_str) {
   if (min_str.ptr() != NULL) {
     if (min_str.length() != 0) {
       end_buf_ = new UInt8[min_str.length()];
-      std::memcpy(end_buf_, min_str.ptr(), min_str.length());
+      grn_memcpy(end_buf_, min_str.ptr(), min_str.length());
       end_str_.assign(end_buf_, min_str.length());
     }
   }
