@@ -28,10 +28,6 @@
 # include <share.h>
 #endif /* WIN32 */
 
-#ifdef WIN32
-# define fileno(file) _fileno(file)
-#endif
-
 static const char *log_level_names[] = {
   "none",
   "emergency",
@@ -127,6 +123,7 @@ rotate_log_file(grn_ctx *ctx, const char *current_path)
   rename(current_path, rotated_path);
 }
 
+static grn_bool logger_inited = GRN_FALSE;
 static char *default_logger_path = NULL;
 static FILE *default_logger_file = NULL;
 static grn_critical_section default_logger_lock;
@@ -149,7 +146,7 @@ default_logger_log(grn_ctx *ctx, grn_log_level level,
       default_logger_size = 0;
       if (default_logger_file) {
         struct stat stat;
-        if (fstat(fileno(default_logger_file), &stat) != -1) {
+        if (fstat(grn_fileno(default_logger_file), &stat) != -1) {
           default_logger_size = stat.st_size;
         }
       }
@@ -262,6 +259,10 @@ grn_default_logger_get_flags(void)
 void
 grn_default_logger_set_path(const char *path)
 {
+  if (logger_inited) {
+    CRITICAL_SECTION_ENTER(default_logger_lock);
+  }
+
   if (default_logger_path) {
     free(default_logger_path);
   }
@@ -271,6 +272,10 @@ grn_default_logger_set_path(const char *path)
   } else {
     default_logger_path = NULL;
   }
+
+  if (logger_inited) {
+    CRITICAL_SECTION_LEAVE(default_logger_lock);
+  }
 }
 
 const char *
@@ -420,10 +425,10 @@ grn_logger_putv(grn_ctx *ctx,
     }
     if (current_logger.flags & GRN_LOG_LOCATION) {
       grn_snprintf(lbuf, LBUFSIZE, LBUFSIZE,
-                   "%d %s:%d %s()", getpid(), file, line, func);
+                   "%d %s:%d %s()", grn_getpid(), file, line, func);
     } else if (current_logger.flags & GRN_LOG_PID) {
       grn_snprintf(lbuf, LBUFSIZE, LBUFSIZE,
-                   "%d", getpid());
+                   "%d", grn_getpid());
     } else {
       lbuf[0] = '\0';
     }
@@ -439,6 +444,8 @@ grn_logger_init(void)
   if (!current_logger.log) {
     current_logger = default_logger;
   }
+
+  logger_inited = GRN_TRUE;
 }
 
 void
@@ -450,9 +457,12 @@ grn_logger_fin(grn_ctx *ctx)
     default_logger_path = NULL;
   }
   CRITICAL_SECTION_FIN(default_logger_lock);
+
+  logger_inited = GRN_FALSE;
 }
 
 
+static grn_bool query_logger_inited = GRN_FALSE;
 static char *default_query_logger_path = NULL;
 static FILE *default_query_logger_file = NULL;
 static grn_critical_section default_query_logger_lock;
@@ -525,7 +535,7 @@ default_query_logger_log(grn_ctx *ctx, unsigned int flag,
       default_query_logger_size = 0;
       if (default_query_logger_file) {
         struct stat stat;
-        if (fstat(fileno(default_query_logger_file), &stat) != -1) {
+        if (fstat(grn_fileno(default_query_logger_file), &stat) != -1) {
           default_query_logger_size = stat.st_size;
         }
       }
@@ -617,6 +627,10 @@ grn_default_query_logger_get_flags(void)
 void
 grn_default_query_logger_set_path(const char *path)
 {
+  if (query_logger_inited) {
+    CRITICAL_SECTION_ENTER(default_query_logger_lock);
+  }
+
   if (default_query_logger_path) {
     free(default_query_logger_path);
   }
@@ -626,6 +640,10 @@ grn_default_query_logger_set_path(const char *path)
   } else {
     default_query_logger_path = NULL;
   }
+
+  if (query_logger_inited) {
+    CRITICAL_SECTION_LEAVE(default_query_logger_lock);
+  }
 }
 
 const char *
@@ -767,6 +785,8 @@ grn_query_logger_init(void)
 {
   current_query_logger = default_query_logger;
   CRITICAL_SECTION_INIT(default_query_logger_lock);
+
+  query_logger_inited = GRN_TRUE;
 }
 
 void
@@ -778,6 +798,8 @@ grn_query_logger_fin(grn_ctx *ctx)
     default_query_logger_path = NULL;
   }
   CRITICAL_SECTION_FIN(default_query_logger_lock);
+
+  query_logger_inited = GRN_FALSE;
 }
 
 void
