@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013 Brazil
+  Copyright(C) 2013-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,23 +16,18 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_MRB_ACCESSOR_H
-#define GRN_MRB_ACCESSOR_H
+#pragma once
 
-#include "../ctx.h"
-#include "../db.h"
+#include "../grn_ctx.h"
+#include "../grn_db.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 void grn_mrb_accessor_init(grn_ctx *ctx);
-#ifdef GRN_WITH_MRUBY
-mrb_value mrb_grn_accessor_new(mrb_state *mrb, grn_accessor *accessor);
-#endif
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* GRN_MRB_ACCESSOR_H */
