@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013-2014 Brazil
+  Copyright(C) 2013-2017 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -17,6 +17,7 @@
 */
 
 #include "../grn_ctx_impl.h"
+#include "../grn_proc.h"
 
 #ifdef GRN_WITH_MRUBY
 #include <mruby.h>
@@ -28,6 +29,24 @@
 #include "mrb_bulk.h"
 #include "mrb_converter.h"
 
+static mrb_value
+mrb_grn_column_class_parse_flags(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  char *error_message_tag;
+  char *flags_text;
+  mrb_int flags_text_size;
+  grn_column_flags flags;
+
+  mrb_get_args(mrb, "zs", &error_message_tag, &flags_text, &flags_text_size);
+
+  flags = grn_proc_column_parse_flags(ctx,
+                                      error_message_tag,
+                                      flags_text,
+                                      flags_text + flags_text_size);
+  return mrb_fixnum_value(flags);
+}
+
 static mrb_value
 mrb_grn_column_array_reference(mrb_state *mrb, mrb_value self)
 {
@@ -91,6 +110,16 @@ mrb_grn_column_is_locked(mrb_state *mrb, mrb_value self)
   return mrb_bool_value(is_locked != 0);
 }
 
+static mrb_value
+mrb_grn_column_clear_lock(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+
+  grn_obj_clear_lock(ctx, DATA_PTR(self));
+  grn_mrb_ctx_check(mrb);
+  return mrb_nil_value();
+}
+
 static mrb_value
 mrb_grn_column_get_table(mrb_state *mrb, mrb_value self)
 {
@@ -105,6 +134,18 @@ mrb_grn_column_get_table(mrb_state *mrb, mrb_value self)
   return grn_mrb_value_from_grn_obj(mrb, table);
 }
 
+static mrb_value
+mrb_grn_column_truncate(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  grn_obj *column;
+
+  column = DATA_PTR(self);
+  grn_column_truncate(ctx, column);
+  grn_mrb_ctx_check(mrb);
+  return mrb_nil_value();
+}
+
 void
 grn_mrb_column_init(grn_ctx *ctx)
 {
@@ -117,6 +158,9 @@ grn_mrb_column_init(grn_ctx *ctx)
   klass = mrb_define_class_under(mrb, module, "Column", object_class);
   MRB_SET_INSTANCE_TT(klass, MRB_TT_DATA);
 
+  mrb_define_class_method(mrb, klass, "parse_flags",
+                          mrb_grn_column_class_parse_flags, MRB_ARGS_REQ(2));
+
   mrb_define_method(mrb, klass, "[]",
                     mrb_grn_column_array_reference, MRB_ARGS_REQ(1));
 
@@ -129,8 +173,13 @@ grn_mrb_column_init(grn_ctx *ctx)
 
   mrb_define_method(mrb, klass, "locked?",
                     mrb_grn_column_is_locked, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "clear_lock",
+                    mrb_grn_column_clear_lock, MRB_ARGS_NONE());
 
   mrb_define_method(mrb, klass, "table",
                     mrb_grn_column_get_table, MRB_ARGS_NONE());
+
+  mrb_define_method(mrb, klass, "truncate",
+                    mrb_grn_column_truncate, MRB_ARGS_NONE());
 }
 #endif
