@@ -176,6 +176,16 @@ mrb_grn_table_is_locked(mrb_state *mrb, mrb_value self)
   return mrb_bool_value(is_locked != 0);
 }
 
+static mrb_value
+mrb_grn_table_clear_lock(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  grn_obj_clear_lock(ctx, DATA_PTR(self));
+  grn_mrb_ctx_check(mrb);
+
+  return mrb_nil_value();
+}
+
 static mrb_value
 mrb_grn_table_get_size(mrb_state *mrb, mrb_value self)
 {
@@ -375,6 +385,18 @@ mrb_grn_table_delete(mrb_state *mrb, mrb_value self)
   return mrb_nil_value();
 }
 
+static mrb_value
+mrb_grn_table_truncate(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  grn_obj *table;
+
+  table = DATA_PTR(self);
+  grn_table_truncate(ctx, table);
+  grn_mrb_ctx_check(mrb);
+  return mrb_nil_value();
+}
+
 static mrb_value
 mrb_grn_table_apply_expression(mrb_state *mrb, mrb_value self)
 {
@@ -454,6 +476,8 @@ grn_mrb_table_init(grn_ctx *ctx)
 
   mrb_define_method(mrb, klass, "locked?",
                     mrb_grn_table_is_locked, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "clear_lock",
+                    mrb_grn_table_clear_lock, MRB_ARGS_NONE());
 
   mrb_define_method(mrb, klass, "size",
                     mrb_grn_table_get_size, MRB_ARGS_NONE());
@@ -470,6 +494,9 @@ grn_mrb_table_init(grn_ctx *ctx)
   mrb_define_method(mrb, klass, "delete",
                     mrb_grn_table_delete, MRB_ARGS_REQ(1));
 
+  mrb_define_method(mrb, klass, "truncate",
+                    mrb_grn_table_truncate, MRB_ARGS_NONE());
+
   mrb_define_method(mrb, klass, "apply_expression",
                     mrb_grn_table_apply_expression, MRB_ARGS_REQ(2));
   mrb_define_method(mrb, klass, "apply_window_function_raw",
