@@ -15,6 +15,7 @@ require "variable_size_column"
 require "index_column"
 require "accessor"
 require "command"
+require "command_input"
 require "table_cursor"
 require "index_cursor"
 
@@ -24,4 +25,14 @@ require "plugin_loader"
 
 require "eval_context"
 
+require "labeled_arguments"
+
 require "command_line_parser"
+
+load_path = ENV["GRN_RUBY_LOAD_PATH"]
+if load_path
+  load_path.split(File::PATH_SEPARATOR).each do |path|
+    next if path.empty?
+    $LOAD_PATH << path
+  end
+end
