@@ -12,7 +12,9 @@ module Groonga
     attr_accessor :flags
     attr_accessor :max_interval
     attr_accessor :similarity_threshold
+    attr_accessor :quorum_threshold
     attr_accessor :start_position
+    attr_accessor :weight
     def initialize(start)
       @start = start
       @end = 0
@@ -24,7 +26,9 @@ module Groonga
       @flags = ScanInfo::Flags::PUSH
       @max_interval = nil
       @similarity_threshold = nil
+      @quorum_threshold = nil
       @start_position = nil
+      @weight = 0
     end
 
     def match_resolve_index
@@ -32,15 +36,20 @@ module Groonga
         match_near_resolve_index
       elsif similar_search?
         match_similar_resolve_index
+      elsif quorum_match?
+        match_quorum_resolve_index
       else
         match_generic_resolve_index
       end
     end
 
     def call_relational_resolve_indexes
-      # better index resolving framework for functions should be implemented
-      @args.each do |arg|
-        call_relational_resolve_index(arg)
+      procedure, *args = *@args
+      return unless procedure.selector?
+
+      selector_op = procedure.selector_operator
+      args.each do |arg|
+        call_relational_resolve_index(arg, selector_op)
       end
     end
 
@@ -91,6 +100,29 @@ module Groonga
       self.similarity_threshold = @args[2].value
     end
 
+    def quorum_match?
+      @op == Operator::QUORUM and @args.size == 3
+    end
+
+    def match_quorum_resolve_index
+      arg = @args[0]
+      case arg
+      when Expression
+        match_resolve_index_expression(arg)
+      when Accessor
+        match_resolve_index_accessor(arg)
+      when Indexable
+        match_resolve_index_indexable(arg)
+      else
+        message =
+          "The first argument of QUORUM must be Expression, Accessor or Indexable: #{arg.class}"
+        raise ErrorMesesage, message
+      end
+
+      self.query = @args[1]
+      self.quorum_threshold = @args[2].value
+    end
+
     def match_generic_resolve_index
       @args.each do |arg|
         case arg
@@ -194,13 +226,12 @@ module Groonga
           end
           weight, offset = codes[i].weight
           i += offset
-          search_index = ScanInfoSearchIndex.new(index_info.index,
-                                                 index_info.section_id,
-                                                 weight,
-                                                 scorer,
-                                                 expression,
-                                                 scorer_args_expr_offset)
-          @search_indexes << search_index
+          put_search_index(index_info.index,
+                           index_info.section_id,
+                           weight,
+                           scorer,
+                           expression,
+                           scorer_args_expr_offset)
         end
       when Table
         raise ErrorMessage, "invalid match target: <#{value.name}>"
@@ -285,32 +316,35 @@ module Groonga
       end
     end
 
-    def call_relational_resolve_index(object)
+    def call_relational_resolve_index(object, selector_op)
       case object
       when Accessor
-        call_relational_resolve_index_accessor(object)
+        call_relational_resolve_index_accessor(object, selector_op)
       when Bulk
         self.query = object
       when Indexable
-        call_relational_resolve_index_indexable(object)
+        call_relational_resolve_index_indexable(object, selector_op)
       end
     end
 
-    def call_relational_resolve_index_indexable(indexable)
-      index_info = indexable.find_index(op)
+    def call_relational_resolve_index_indexable(indexable, selector_op)
+      index_info = indexable.find_index(selector_op)
       return if index_info.nil?
       put_search_index(index_info.index, index_info.section_id, 1)
     end
 
-    def call_relational_resolve_index_accessor(accessor)
+    def call_relational_resolve_index_accessor(accessor, selector_op)
       self.flags |= ScanInfo::Flags::ACCESSOR
-      index_info = accessor.find_index(op)
+      index_info = accessor.find_index(selector_op)
       return if index_info.nil?
       put_search_index(index_info.index, index_info.section_id, 1)
     end
 
-    def put_search_index(index, section_id, weight)
-      search_index = ScanInfoSearchIndex.new(index, section_id, weight)
+    def put_search_index(index, section_id, weight, *args)
+      search_index = ScanInfoSearchIndex.new(index,
+                                             section_id,
+                                             weight + @weight,
+                                             *args)
       @search_indexes << search_index
     end
   end
