@@ -20,6 +20,7 @@
 
 #include "grn_normalizer.h"
 #include "grn_string.h"
+#include "grn_nfkc.h"
 #include <groonga/normalizer.h>
 #include <groonga/tokenizer.h>
 
@@ -77,7 +78,7 @@ static unsigned char symbol[] = {
   '$', 0, 0, '%', '#', '&', '*', '@', 0, 0, 0, 0, 0, 0, 0, 0
 };
 
-inline static grn_obj *
+grn_inline static grn_obj *
 eucjp_normalize(grn_ctx *ctx, grn_string *nstr)
 {
   static uint16_t hankana[] = {
@@ -322,7 +323,7 @@ eucjp_normalize(grn_ctx *ctx, grn_string *nstr)
   return NULL;
 }
 
-inline static grn_obj *
+grn_inline static grn_obj *
 sjis_normalize(grn_ctx *ctx, grn_string *nstr)
 {
   static uint16_t hankana[] = {
@@ -559,10 +560,7 @@ sjis_normalize(grn_ctx *ctx, grn_string *nstr)
 }
 
 #ifdef GRN_WITH_NFKC
-const char *grn_nfkc_map1(const unsigned char *str);
-const char *grn_nfkc_map2(const unsigned char *prefix, const unsigned char *suffix);
-
-static inline int
+static grn_inline int
 grn_str_charlen_utf8(grn_ctx *ctx, const unsigned char *str, const unsigned char *end)
 {
   /* MEMO: This function allows non-null-terminated string as str. */
@@ -612,7 +610,7 @@ grn_str_charlen_utf8(grn_ctx *ctx, const unsigned char *str, const unsigned char
   return 0;
 }
 
-inline static grn_obj *
+grn_inline static grn_obj *
 utf8_normalize(grn_ctx *ctx, grn_string *nstr)
 {
   int16_t *ch;
@@ -661,13 +659,13 @@ utf8_normalize(grn_ctx *ctx, grn_string *nstr)
                                              GRN_ENC_UTF8)) {
       continue;
     }
-    if ((p = (unsigned char *)grn_nfkc_map1(s))) {
+    if ((p = (unsigned char *)grn_nfkc_decompose(s))) {
       pe = p + strlen((char *)p);
     } else {
       p = s;
       pe = p + ls;
     }
-    if (d_ && (p2 = (unsigned char *)grn_nfkc_map2(d_, p))) {
+    if (d_ && (p2 = (unsigned char *)grn_nfkc_compose(d_, p))) {
       p = p2;
       pe = p + strlen((char *)p);
       if (cp) { cp--; }
@@ -755,7 +753,7 @@ utf8_normalize(grn_ctx *ctx, grn_string *nstr)
 }
 #endif /* GRN_WITH_NFKC */
 
-inline static grn_obj *
+grn_inline static grn_obj *
 ascii_normalize(grn_ctx *ctx, grn_string *nstr)
 {
   int16_t *ch;
@@ -857,7 +855,7 @@ ascii_normalize(grn_ctx *ctx, grn_string *nstr)
 }
 
 /* use cp1252 as latin1 */
-inline static grn_obj *
+grn_inline static grn_obj *
 latin1_normalize(grn_ctx *ctx, grn_string *nstr)
 {
   int16_t *ch;
@@ -992,7 +990,7 @@ latin1_normalize(grn_ctx *ctx, grn_string *nstr)
   return NULL;
 }
 
-inline static grn_obj *
+grn_inline static grn_obj *
 koi8r_normalize(grn_ctx *ctx, grn_string *nstr)
 {
   int16_t *ch;
