@@ -108,7 +108,7 @@ typedef struct {
   score_heap_node *nodes;
 } score_heap;
 
-static inline score_heap *
+static grn_inline score_heap *
 score_heap_open(grn_ctx *ctx, int max)
 {
   score_heap *h = GRN_PLUGIN_MALLOC(ctx, sizeof(score_heap));
@@ -123,7 +123,7 @@ score_heap_open(grn_ctx *ctx, int max)
   return h;
 }
 
-static inline grn_bool
+static grn_inline grn_bool
 score_heap_push(grn_ctx *ctx, score_heap *h, grn_id id, uint32_t score)
 {
   int n, n2;
@@ -152,7 +152,7 @@ score_heap_push(grn_ctx *ctx, score_heap *h, grn_id id, uint32_t score)
   return GRN_TRUE;
 }
 
-static inline void
+static grn_inline void
 score_heap_close(grn_ctx *ctx, score_heap *h)
 {
   GRN_PLUGIN_FREE(ctx, h->nodes);
@@ -190,8 +190,17 @@ sequential_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *column, grn_obj *
     while ((id = grn_table_cursor_next(ctx, tc))) {
       unsigned int distance = 0;
       grn_obj *domain;
+      grn_id record_id;
+
+      if (op == GRN_OP_AND) {
+        grn_id *key;
+        grn_table_cursor_get_key(ctx, tc, (void **)&key);
+        record_id = *key;
+      } else {
+        record_id = id;
+      }
       GRN_BULK_REWIND(&value);
-      grn_obj_get_value(ctx, column, id, &value);
+      grn_obj_get_value(ctx, column, record_id, &value);
       domain = grn_ctx_at(ctx, ((&value))->header.domain);
       if ((&(value))->header.type == GRN_VECTOR) {
         n = grn_vector_size(ctx, &value);
@@ -207,7 +216,7 @@ sequential_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *column, grn_obj *
                                           (char *)vector_value,
                                           (char *)vector_value + length, flags);
             if (distance <= max_distance) {
-              score_heap_push(ctx, heap, id, distance);
+              score_heap_push(ctx, heap, record_id, distance);
               break;
             }
           }
@@ -228,7 +237,7 @@ sequential_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *column, grn_obj *
             distance = calc_edit_distance(ctx, sx, ex,
                                           key_name, key_name + key_length, flags);
             if (distance <= max_distance) {
-              score_heap_push(ctx, heap, id, distance);
+              score_heap_push(ctx, heap, record_id, distance);
               break;
             }
           }
@@ -246,7 +255,7 @@ sequential_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *column, grn_obj *
             distance = calc_edit_distance(ctx, sx, ex,
                                           key_name, key_name + key_length, flags);
             if (distance <= max_distance) {
-              score_heap_push(ctx, heap, id, distance);
+              score_heap_push(ctx, heap, record_id, distance);
             }
           }
         } else {
@@ -257,7 +266,7 @@ sequential_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *column, grn_obj *
                                           GRN_TEXT_VALUE(&value),
                                           GRN_BULK_CURR(&value), flags);
             if (distance <= max_distance) {
-              score_heap_push(ctx, heap, id, distance);
+              score_heap_push(ctx, heap, record_id, distance);
             }
           }
         }
@@ -315,51 +324,58 @@ selector_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *index,
 
   if (nargs == 4) {
     grn_obj *options = args[3];
-    grn_hash_cursor *cursor;
-    void *key;
-    grn_obj *value;
-    int key_size;
 
-    if (options->header.type != GRN_TABLE_HASH_KEY) {
+    switch (options->header.type) {
+    case GRN_BULK :
+      max_distance = GRN_UINT32_VALUE(options);
+      break;
+    case GRN_TABLE_HASH_KEY :
+      {
+        grn_hash_cursor *cursor;
+        void *key;
+        grn_obj *value;
+        int key_size;
+        cursor = grn_hash_cursor_open(ctx, (grn_hash *)options,
+                                      NULL, 0, NULL, 0,
+                                      0, -1, 0);
+        if (!cursor) {
+          GRN_PLUGIN_ERROR(ctx, GRN_NO_MEMORY_AVAILABLE,
+                           "fuzzy_search(): couldn't open cursor");
+          goto exit;
+        }
+        while (grn_hash_cursor_next(ctx, cursor) != GRN_ID_NIL) {
+          grn_hash_cursor_get_key_value(ctx, cursor, &key, &key_size,
+                                        (void **)&value);
+
+          if (key_size == 12 && !memcmp(key, "max_distance", 12)) {
+            max_distance = GRN_UINT32_VALUE(value);
+          } else if (key_size == 13 && !memcmp(key, "prefix_length", 13)) {
+            prefix_length = GRN_UINT32_VALUE(value);
+          } else if (key_size == 13 && !memcmp(key, "max_expansion", 13)) {
+            max_expansion = GRN_UINT32_VALUE(value);
+          } else if (key_size == 18 && !memcmp(key, "with_transposition", 18)) {
+            if (GRN_BOOL_VALUE(value)) {
+              flags |= GRN_TABLE_FUZZY_SEARCH_WITH_TRANSPOSITION;
+            }
+          } else {
+            GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
+                             "invalid option name: <%.*s>",
+                             key_size, (char *)key);
+            grn_hash_cursor_close(ctx, cursor);
+            goto exit;
+          }
+        }
+        grn_hash_cursor_close(ctx, cursor);
+      }
+      break;
+    default :
       GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
                        "fuzzy_search(): "
-                       "3rd argument must be object literal: <%.*s>",
+                       "3rd argument must be integer or object literal: <%.*s>",
                        (int)GRN_TEXT_LEN(options),
                        GRN_TEXT_VALUE(options));
       goto exit;
     }
-
-    cursor = grn_hash_cursor_open(ctx, (grn_hash *)options,
-                                  NULL, 0, NULL, 0,
-                                  0, -1, 0);
-    if (!cursor) {
-      GRN_PLUGIN_ERROR(ctx, GRN_NO_MEMORY_AVAILABLE,
-                       "fuzzy_search(): couldn't open cursor");
-      goto exit;
-    }
-    while (grn_hash_cursor_next(ctx, cursor) != GRN_ID_NIL) {
-      grn_hash_cursor_get_key_value(ctx, cursor, &key, &key_size,
-                                    (void **)&value);
-
-      if (key_size == 12 && !memcmp(key, "max_distance", 12)) {
-        max_distance = GRN_UINT32_VALUE(value);
-      } else if (key_size == 13 && !memcmp(key, "prefix_length", 13)) {
-        prefix_length = GRN_UINT32_VALUE(value);
-      } else if (key_size == 13 && !memcmp(key, "max_expansion", 13)) {
-        max_expansion = GRN_UINT32_VALUE(value);
-      } else if (key_size == 18 && !memcmp(key, "with_transposition", 18)) {
-        if (GRN_BOOL_VALUE(value)) {
-          flags |= GRN_TABLE_FUZZY_SEARCH_WITH_TRANSPOSITION;
-        }
-      } else {
-        GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
-                         "invalid option name: <%.*s>",
-                         key_size, (char *)key);
-        grn_hash_cursor_close(ctx, cursor);
-        goto exit;
-      }
-    }
-    grn_hash_cursor_close(ctx, cursor);
   }
 
   if (index) {
@@ -447,4 +463,5 @@ grn_proc_init_fuzzy_search(grn_ctx *ctx)
                                   GRN_PROC_FUNCTION,
                                   NULL, NULL, NULL, 0, NULL);
   grn_proc_set_selector(ctx, selector_proc, selector_fuzzy_search);
+  grn_proc_set_selector_operator(ctx, selector_proc, GRN_OP_FUZZY);
 }
