@@ -22,6 +22,7 @@
 #include "grn_util.h"
 #include "grn_string.h"
 #include "grn_expr.h"
+#include "grn_load.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -41,19 +42,6 @@ grn_normalize_offset_and_limit(grn_ctx *ctx, int size, int *p_offset, int *p_lim
   int offset = *p_offset;
   int limit = *p_limit;
 
-  if (offset < 0) {
-    offset += size;
-    if (offset < 0) {
-      *p_offset = 0;
-      *p_limit = 0;
-      return GRN_TOO_SMALL_OFFSET;
-    }
-  } else if (offset != 0 && offset >= size) {
-    *p_offset = 0;
-    *p_limit = 0;
-    return GRN_TOO_LARGE_OFFSET;
-  }
-
   if (limit < 0) {
     limit += size + 1;
     if (limit < 0) {
@@ -65,6 +53,24 @@ grn_normalize_offset_and_limit(grn_ctx *ctx, int size, int *p_offset, int *p_lim
     limit = size;
   }
 
+  if (offset < 0) {
+    offset += size;
+    if (offset < 0) {
+      if (limit + offset >= 0) {
+        limit += offset;
+        offset = 0;
+      } else {
+        *p_offset = 0;
+        *p_limit = 0;
+        return GRN_TOO_SMALL_OFFSET;
+      }
+    }
+  } else if (offset != 0 && offset >= size) {
+    *p_offset = 0;
+    *p_limit = 0;
+    return GRN_TOO_LARGE_OFFSET;
+  }
+
   /* At this point, offset and limit must be zero or positive. */
   end = offset + limit;
   if (end > size) {
@@ -247,6 +253,39 @@ grn_inspect_type(grn_ctx *ctx, grn_obj *buf, unsigned char type)
   return buf;
 }
 
+
+grn_obj *
+grn_inspect_query_log_flags(grn_ctx *ctx, grn_obj *buffer, unsigned int flags)
+{
+  grn_bool have_content = GRN_FALSE;
+
+  if (flags == GRN_QUERY_LOG_NONE) {
+    GRN_TEXT_PUTS(ctx, buffer, "NONE");
+    return buffer;
+  }
+
+#define CHECK_FLAG(NAME) do {                   \
+    if (flags & GRN_QUERY_LOG_ ## NAME) {       \
+      if (have_content) {                       \
+        GRN_TEXT_PUTS(ctx, buffer, "|");        \
+      }                                         \
+      GRN_TEXT_PUTS(ctx, buffer, #NAME);        \
+      have_content = GRN_TRUE;                  \
+    }                                           \
+  } while (GRN_FALSE)
+
+  CHECK_FLAG(COMMAND);
+  CHECK_FLAG(RESULT_CODE);
+  CHECK_FLAG(DESTINATION);
+  CHECK_FLAG(CACHE);
+  CHECK_FLAG(SIZE);
+  CHECK_FLAG(SCORE);
+
+#undef CHECK_FALG
+
+  return buffer;
+}
+
 static grn_rc
 grn_proc_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 {
@@ -1449,6 +1488,30 @@ grn_inspect_indented(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj,
   return buffer;
 }
 
+grn_obj *
+grn_inspect_limited(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
+{
+  grn_obj sub_buffer;
+  unsigned int max_size = GRN_CTX_MSGSIZE / 2;
+
+  GRN_TEXT_INIT(&sub_buffer, 0);
+  grn_inspect(ctx, &sub_buffer, obj);
+  if (GRN_TEXT_LEN(&sub_buffer) > max_size) {
+    GRN_TEXT_PUT(ctx, buffer, GRN_TEXT_VALUE(&sub_buffer), max_size);
+    GRN_TEXT_PUTS(ctx, buffer, "...(");
+    grn_text_lltoa(ctx, buffer, GRN_TEXT_LEN(&sub_buffer));
+    GRN_TEXT_PUTS(ctx, buffer, ")");
+  } else {
+    GRN_TEXT_PUT(ctx,
+                 buffer,
+                 GRN_TEXT_VALUE(&sub_buffer),
+                 GRN_TEXT_LEN(&sub_buffer));
+  }
+  GRN_OBJ_FIN(ctx, &sub_buffer);
+
+  return buffer;
+}
+
 void
 grn_p(grn_ctx *ctx, grn_obj *obj)
 {
@@ -1549,33 +1612,37 @@ grn_path_exist(const char *path)
   return stat(path, &status) == 0;
 }
 
-#ifdef WIN32
-UINT
-grn_windows_encoding_to_code_page(grn_encoding encoding)
+/* todo : refine */
+/*
+ * grn_tokenize splits a string into at most buf_size tokens and
+ * returns the number of tokens. The ending address of each token is
+ * written into tokbuf. Delimiters are ' ' and ','.
+ * Then, the address to the remaining is set to rest.
+ */
+int
+grn_tokenize(const char *str, size_t str_len,
+             const char **tokbuf, int buf_size,
+             const char **rest)
 {
-  UINT code_page;
-
-  switch (encoding) {
-  case GRN_ENC_EUC_JP :
-    code_page = 20932;
-    break;
-  case GRN_ENC_UTF8 :
-    code_page = CP_UTF8;
-    break;
-  case GRN_ENC_SJIS :
-    code_page = 932;
-    break;
-  case GRN_ENC_LATIN1 :
-    code_page = 1252;
-    break;
-  case GRN_ENC_KOI8R :
-    code_page = 20866;
-    break;
-  default :
-    code_page = CP_ACP;
-    break;
+  const char **tok = tokbuf, **tok_end = tokbuf + buf_size;
+  if (buf_size > 0) {
+    const char *str_end = str + str_len;
+    while (str < str_end && (' ' == *str || ',' == *str)) { str++; }
+    for (;;) {
+      if (str == str_end) {
+        *tok++ = str;
+        break;
+      }
+      if (' ' == *str || ',' == *str) {
+        /* *str = '\0'; */
+        *tok++ = str;
+        if (tok == tok_end) { break; }
+        do { str++; } while (str < str_end && (' ' == *str || ',' == *str));
+      } else {
+        str++;
+      }
+    }
   }
-
-  return code_page;
+  if (rest) { *rest = str; }
+  return tok - tokbuf;
 }
-#endif /* WIN32 */
