@@ -15,6 +15,7 @@ module Groonga
                  "limit",
                  "output_columns",
                  "use_range_index",
+                 "post_filter",
                ])
 
       def run_body(input)
@@ -71,6 +72,9 @@ module Groonga
         key << "#{input[:limit]}\0"
         key << "#{input[:output_columns]}\0"
         key << "#{input[:use_range_index]}\0"
+        key << "#{input[:post_filter]}\0"
+        dynamic_columns = DynamicColumns.parse(input)
+        key << dynamic_columns.cache_key
         key
       end
 
@@ -81,11 +85,14 @@ module Groonga
         attr_reader :filter
         attr_reader :offset
         attr_reader :limit
+        attr_reader :dynamic_columns
         attr_accessor :current_offset
         attr_accessor :current_limit
         attr_reader :result_sets
         attr_reader :unsorted_result_sets
+        attr_reader :temporary_tables
         attr_reader :threshold
+        attr_reader :post_filter
         def initialize(input)
           @input = input
           @use_range_index = parse_use_range_index(@input[:use_range_index])
@@ -94,6 +101,7 @@ module Groonga
           @filter = @input[:filter]
           @offset = (@input[:offset] || 0).to_i
           @limit = (@input[:limit] || 10).to_i
+          @dynamic_columns = DynamicColumns.parse(@input)
 
           @current_offset = @offset
           @current_limit = @limit
@@ -101,7 +109,11 @@ module Groonga
           @result_sets = []
           @unsorted_result_sets = []
 
+          @temporary_tables = []
+
           @threshold = compute_threshold
+
+          @post_filter = @input[:post_filter]
         end
 
         def close
@@ -111,6 +123,10 @@ module Groonga
           @result_sets.each do |result_set|
             result_set.close if result_set.temporary?
           end
+
+          @temporary_tables.each do |table|
+            table.close
+          end
         end
 
         private
@@ -179,6 +195,12 @@ module Groonga
           if @context.result_sets.empty?
             result_set = HashTable.create(:flags => ObjectFlags::WITH_SUBREC,
                                           :key_type => first_shard.table)
+            @context.dynamic_columns.each_initial do |dynamic_column|
+              dynamic_column.apply(result_set)
+            end
+            @context.dynamic_columns.each_filtered do |dynamic_column|
+              dynamic_column.apply(result_set)
+            end
             @context.result_sets << result_set
           end
         end
@@ -190,7 +212,10 @@ module Groonga
           @shard = shard
           @shard_range = shard_range
 
+          @target_table = @shard.table
+
           @filter = @context.filter
+          @post_filter = @context.post_filter
           @result_sets = @context.result_sets
           @unsorted_result_sets = @context.unsorted_result_sets
 
@@ -201,7 +226,7 @@ module Groonga
 
         def execute
           return if @cover_type == :none
-          return if @shard.table.empty?
+          return if @target_table.empty?
 
           shard_key = @shard.key
           if shard_key.nil?
@@ -211,8 +236,8 @@ module Groonga
           end
 
           expression_builder = RangeExpressionBuilder.new(shard_key,
-                                                          @target_range,
-                                                          @filter)
+                                                          @target_range)
+          expression_builder.filter = @filter
 
           index_info = shard_key.find_index(Operator::LESS)
           if index_info
@@ -224,6 +249,24 @@ module Groonga
             range_index = nil
           end
 
+          @context.dynamic_columns.each_initial do |dynamic_column|
+            if @target_table == @shard.table
+              if @cover_type == :all
+                @target_table = @target_table.select_all
+              else
+                expression_builder.filter = nil
+                @target_table = create_expression(@target_table) do |expression|
+                  expression_builder.build(expression, @shard_range)
+                  @target_table.select(expression)
+                end
+                @cover_type = :all
+                expression_builder.filter = @filter
+              end
+              @context.temporary_tables << @target_table
+            end
+            dynamic_column.apply(@target_table)
+          end
+
           execute_filter(range_index, expression_builder)
         end
 
@@ -273,11 +316,20 @@ module Groonga
                                           __LINE__, __method__)
           end
 
+          unless @context.dynamic_columns.empty?
+            reason = "dynamic columns are used"
+            return decide_use_range_index(false, reason, __LINE__, __method__)
+          end
+
+          unless @context.post_filter.nil?
+            reason = "post_filter is used"
+            return decide_use_range_index(false, reason, __LINE__, __method__)
+          end
+
           current_limit = @context.current_limit
           if current_limit < 0
             reason = "limit is negative: <#{current_limit}>"
-            return decide_use_range_index(false, reason,
-                                          __LINE__, __method__)
+            return decide_use_range_index(false, reason, __LINE__, __method__)
           end
 
           required_n_records = @context.current_offset + current_limit
@@ -449,9 +501,9 @@ module Groonga
         end
 
         def filter_shard_all(range_index, expression_builder)
-          table = @shard.table
+          table = @target_table
           if @filter.nil?
-            if table.size <= @context.current_offset
+            if @post_filter.nil? and table.size <= @context.current_offset
               @context.current_offset -= table.size
               return
             end
@@ -593,7 +645,7 @@ module Groonga
         end
 
         def filter_table
-          table = @shard.table
+          table = @target_table
           create_expression(table) do |expression|
             yield(expression)
             result_set = table.select(expression)
@@ -601,12 +653,34 @@ module Groonga
           end
         end
 
+        def apply_post_filter(table)
+          create_expression(table) do |expression|
+            expression.parse(@post_filter)
+            table.select(expression)
+          end
+        end
+
         def sort_result_set(result_set)
+          @context.temporary_tables.delete(result_set)
+
           if result_set.empty?
             result_set.close if result_set.temporary?
             return
           end
 
+          @context.dynamic_columns.each_filtered do |dynamic_column|
+            if result_set == @shard.table
+              result_set = result_set.select_all
+            end
+            dynamic_column.apply(result_set)
+          end
+
+          unless @post_filter.nil?
+            filtered_table = result_set
+            result_set = apply_post_filter(filtered_table)
+            @context.temporary_tables << filtered_table
+          end
+
           if result_set.size <= @context.current_offset
             @context.current_offset -= result_set.size
             result_set.close if result_set.temporary?
