@@ -91,7 +91,7 @@ get_mecab_encoding(mecab_t *mecab)
   return encoding;
 }
 
-static inline grn_bool
+static grn_inline grn_bool
 is_delimiter_character(grn_ctx *ctx, const char *character, int character_bytes)
 {
   switch (character_bytes) {
@@ -170,7 +170,7 @@ chunked_tokenize_utf8_chunk(grn_ctx *ctx,
 
   tokenized_chunk_length = strlen(tokenized_chunk);
   if (tokenized_chunk_length >= 1 &&
-      isspace(tokenized_chunk[tokenized_chunk_length - 1])) {
+      isspace((unsigned char)tokenized_chunk[tokenized_chunk_length - 1])) {
     GRN_TEXT_PUT(ctx, &(tokenizer->buf),
                  tokenized_chunk, tokenized_chunk_length - 1);
   } else {
