@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2010-2012 Brazil
+/* Copyright(C) 2010-2015 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -14,11 +14,15 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#include <zmq.h>
+
+/* for grn_str_getopt() */
+#include <grn_str.h>
+#include <grn_msgpack.h>
+
+#include "zmq_compatible.h"
 #include <stdio.h>
 #include <signal.h>
 #include <unistd.h>
-#include <msgpack.h>
 #include <pthread.h>
 #include <groonga.h>
 #include <inttypes.h>
@@ -36,6 +40,15 @@
 
 #define CONST_STR_LEN(x) x, x ? sizeof(x) - 1 : 0
 
+typedef enum {
+  RUN_MODE_NONE   = 0x00,
+  RUN_MODE_USAGE  = 0x01,
+  RUN_MODE_DAEMON = 0x02,
+  RUN_MODE_ERROR  = 0x04
+} run_mode;
+
+#define RUN_MODE_MASK                0x007f
+
 typedef struct {
   const char *db_path;
   const char *send_endpoint;
@@ -129,14 +142,14 @@ load_to_multi_targets(grn_ctx *ctx,
   int _k_len; \
   char _k_buf[GRN_TABLE_MAX_KEY_SIZE]; \
   _k_len = grn_table_get_key(ctx, ref_table, (id), _k_buf, GRN_TABLE_MAX_KEY_SIZE); \
-  msgpack_pack_raw(&pk, _k_len); \
-  msgpack_pack_raw_body(&pk, _k_buf, _k_len); \
+  msgpack_pack_str(&pk, _k_len); \
+  msgpack_pack_str_body(&pk, _k_buf, _k_len); \
 } while (0)
 
 #define PACK_MAP_ITEM(col_name) do { \
   grn_obj _v; \
-  msgpack_pack_raw(&pk, sizeof(#col_name) - 1); \
-  msgpack_pack_raw_body(&pk, CONST_STR_LEN(#col_name)); \
+  msgpack_pack_str(&pk, sizeof(#col_name) - 1); \
+  msgpack_pack_str_body(&pk, #col_name, sizeof(#col_name) - 1); \
   switch (col_##col_name->header.type) { \
   case GRN_COLUMN_FIX_SIZE: \
     GRN_VALUE_FIX_SIZE_INIT(&_v, 0, grn_obj_get_range(ctx, col_##col_name)); \
@@ -155,8 +168,8 @@ load_to_multi_targets(grn_ctx *ctx,
   case GRN_BULK: \
     switch (_v.header.domain) { \
     case GRN_DB_SHORT_TEXT: \
-      msgpack_pack_raw(&pk, GRN_TEXT_LEN(&_v)); \
-      msgpack_pack_raw_body(&pk, GRN_TEXT_VALUE(&_v), GRN_TEXT_LEN(&_v)); \
+      msgpack_pack_str(&pk, GRN_TEXT_LEN(&_v)); \
+      msgpack_pack_str_body(&pk, GRN_TEXT_VALUE(&_v), GRN_TEXT_LEN(&_v)); \
       break; \
     case GRN_DB_INT32: \
       msgpack_pack_int32(&pk, GRN_INT32_VALUE(&_v)); \
@@ -194,7 +207,7 @@ zmq_send_to_httpd(void *zmq_send_sock, void *data, size_t size)
   zmq_msg_t msg;
   if (!zmq_msg_init_size(&msg, size)) {
     memcpy((void *)zmq_msg_data(&msg), data, size);
-    if (zmq_send(zmq_send_sock, &msg, 0)) {
+    if (zmq_msg_send(&msg, zmq_send_sock, 0) == -1) {
       print_error("zmq_send() error");
       return -1;
     }
@@ -255,16 +268,18 @@ send_handler(void *zmq_send_sock, grn_ctx *ctx)
                 msgpack_pack_map(&pk, 8);
 
                 /* ["_key","ShortText"],["last","Time"],["kana","kana"],["freq2","Int32"],["freq","Int32"],["co","pair_all"],["buzz","Int32"],["boost","Int32"] */
-                msgpack_pack_raw(&pk, 6);
-                msgpack_pack_raw_body(&pk, CONST_STR_LEN("target"));
-                msgpack_pack_raw(&pk, name_len);
-                msgpack_pack_raw_body(&pk, name_buf, name_len);
+                msgpack_pack_str(&pk, 6);
+                msgpack_pack_str_body(&pk, "target", strlen("target"));
+                msgpack_pack_str(&pk, name_len);
+                msgpack_pack_str_body(&pk, name_buf, name_len);
 
-                msgpack_pack_raw(&pk, 4);
-                msgpack_pack_raw_body(&pk, CONST_STR_LEN("_key"));
+                msgpack_pack_str(&pk, 4);
+                msgpack_pack_str_body(&pk,
+                                      GRN_COLUMN_NAME_KEY,
+                                      GRN_COLUMN_NAME_KEY_LEN);
                 key_len = grn_table_cursor_get_key(ctx, tc, (void **)&key);
-                msgpack_pack_raw(&pk, key_len);
-                msgpack_pack_raw_body(&pk, key, key_len);
+                msgpack_pack_str(&pk, key_len);
+                msgpack_pack_str_body(&pk, key, key_len);
 
                 PACK_MAP_ITEM(last);
                 PACK_MAP_ITEM(kana);
@@ -324,13 +339,15 @@ send_handler(void *zmq_send_sock, grn_ctx *ctx)
                 msgpack_pack_map(&pk, 7);
                 /* ["_key","UInt64"],["pre","item_all"],["post","item_all"],["freq2","Int32"],["freq1","Int32"],["freq0","Int32"] */
 
-                msgpack_pack_raw(&pk, 6);
-                msgpack_pack_raw_body(&pk, CONST_STR_LEN("target"));
-                msgpack_pack_raw(&pk, name_len);
-                msgpack_pack_raw_body(&pk, name_buf, name_len);
+                msgpack_pack_str(&pk, 6);
+                msgpack_pack_str_body(&pk, "target", strlen("target"));
+                msgpack_pack_str(&pk, name_len);
+                msgpack_pack_str_body(&pk, name_buf, name_len);
 
-                msgpack_pack_raw(&pk, 4);
-                msgpack_pack_raw_body(&pk, CONST_STR_LEN("_key"));
+                msgpack_pack_str(&pk, 4);
+                msgpack_pack_str_body(&pk,
+                                      GRN_COLUMN_NAME_KEY,
+                                      GRN_COLUMN_NAME_KEY_LEN);
                 grn_table_cursor_get_key(ctx, tc, (void **)&key);
                 msgpack_pack_uint64(&pk, *key);
 
@@ -369,7 +386,7 @@ send_to_httpd(void *arg)
         grn_obj *db;
         if ((db = grn_db_open(&ctx, thd->db_path))) {
           uint64_t hwm = 1;
-          zmq_setsockopt(zmq_send_sock, ZMQ_HWM, &hwm, sizeof(uint64_t));
+          zmq_setsockopt(zmq_send_sock, ZMQ_SNDHWM, &hwm, sizeof(uint64_t));
           while (loop) {
             send_handler(zmq_send_sock, &ctx);
           }
@@ -402,25 +419,29 @@ handle_msg(msgpack_object *obj, grn_ctx *ctx, grn_obj *buf)
     int i;
     for (i = 0; i < obj->via.map.size; i++) {
       msgpack_object_kv *kv;
+      msgpack_object *key;
+      msgpack_object *value;
       kv = &(obj->via.map.ptr[i]);
-      if (kv->key.type == MSGPACK_OBJECT_RAW && kv->key.via.raw.size) {
-        switch (kv->key.via.raw.ptr[0]) {
+      key = &(kv->key);
+      value = &(kv->val);
+      if (key->type == MSGPACK_OBJECT_STR && MSGPACK_OBJECT_STR_SIZE(key) > 0) {
+        switch (MSGPACK_OBJECT_STR_PTR(key)[0]) {
         case 'i':
-          if (kv->val.type == MSGPACK_OBJECT_RAW) {
-            client_id_len = kv->val.via.raw.size;
-            client_id = kv->val.via.raw.ptr;
+          if (value->type == MSGPACK_OBJECT_STR) {
+            client_id_len = MSGPACK_OBJECT_STR_SIZE(value);
+            client_id = MSGPACK_OBJECT_STR_PTR(value);
           }
           break;
         case 'q':
-          if (kv->val.type == MSGPACK_OBJECT_RAW) {
-            query_len = kv->val.via.raw.size;
-            query = kv->val.via.raw.ptr;
+          if (value->type == MSGPACK_OBJECT_STR) {
+            query_len = MSGPACK_OBJECT_STR_SIZE(value);
+            query = MSGPACK_OBJECT_STR_PTR(value);
           }
           break;
         case 'l':
-          if (kv->val.type == MSGPACK_OBJECT_RAW) {
-            learn_target_names_len = kv->val.via.raw.size;
-            learn_target_names = kv->val.via.raw.ptr;
+          if (value->type == MSGPACK_OBJECT_STR) {
+            learn_target_names_len = MSGPACK_OBJECT_STR_SIZE(value);
+            learn_target_names = MSGPACK_OBJECT_STR_PTR(value);
           }
           break;
         case 's':
@@ -460,7 +481,7 @@ recv_event_loop(msgpack_zone *mempool, void *zmq_sock, grn_ctx *ctx)
       if (zmq_msg_init(&msg)) {
         print_error("cannot init zmq message.");
       } else {
-        if (zmq_recv(zmq_sock, &msg, 0)) {
+        if (zmq_msg_recv(&msg, zmq_sock, 0) == -1) {
           print_error("cannot recv zmq message.");
         } else {
           msgpack_object obj;
@@ -673,10 +694,19 @@ usage(FILE *output)
           "Usage: groonga-suggest-learner [options...] db_path\n"
           "options:\n"
           "  -r <recv endpoint>: recv endpoint (default: %s)\n"
+          "  --receive-endpoint <recv endpoint>\n"
+          "\n"
           "  -s <send endpoint>: send endpoint (default: %s)\n"
+          "  --send-endpoint <send endpoint>\n"
+          "\n"
           "  -l <log directory>: load from log files made on webserver.\n"
-          "  -d                : daemonize\n",
-          DEFAULT_RECV_ENDPOINT, DEFAULT_SEND_ENDPOINT);
+          "  --log-base-path <log directory>\n"
+          "\n"
+          "  --log-path <path> : output logs to <path>\n"
+          "  --log-level <level> : set log level to <level> (default: %d)\n"
+          "  -d, --daemon      : daemonize\n",
+          DEFAULT_RECV_ENDPOINT, DEFAULT_SEND_ENDPOINT,
+          GRN_LOG_DEFAULT_LEVEL);
 }
 
 static void
@@ -688,54 +718,84 @@ signal_handler(int sig)
 int
 main(int argc, char **argv)
 {
-  int daemon = 0;
-  const char *recv_endpoint = DEFAULT_RECV_ENDPOINT,
-             *send_endpoint = DEFAULT_SEND_ENDPOINT,
-             *load_logfile_name = NULL;
+  run_mode mode = RUN_MODE_NONE;
+  int n_processed_args;
+  const char *recv_endpoint = DEFAULT_RECV_ENDPOINT;
+  const char *send_endpoint = DEFAULT_SEND_ENDPOINT;
+  const char *log_base_path = NULL;
+  const char *db_path = NULL;
 
   /* parse options */
   {
-    int ch;
+    int flags = mode;
+    const char *log_path = NULL;
+    const char *log_level = NULL;
+    static grn_str_getopt_opt opts[] = {
+      {'r', "receive-endpoint", NULL, 0, GETOPT_OP_NONE},
+      {'s', "send-endpoint", NULL, 0, GETOPT_OP_NONE},
+      {'l', "log-base-path", NULL, 0, GETOPT_OP_NONE},
+      {'\0', "log-path", NULL, 0, GETOPT_OP_NONE},
+      {'\0', "log-level", NULL, 0, GETOPT_OP_NONE},
+      {'d', "daemon", NULL, RUN_MODE_DAEMON, GETOPT_OP_UPDATE},
+      {'h', "help", NULL, RUN_MODE_USAGE, GETOPT_OP_UPDATE},
+      {'\0', NULL, NULL, 0, 0}
+    };
+    opts[0].arg = &recv_endpoint;
+    opts[1].arg = &send_endpoint;
+    opts[2].arg = &log_base_path;
+    opts[3].arg = &log_path;
+    opts[4].arg = &log_level;
 
-    while ((ch = getopt(argc, argv, "r:s:dl:")) != -1) {
-      switch(ch) {
-      case 'r':
-        recv_endpoint = optarg;
-        break;
-      case 's':
-        send_endpoint = optarg;
-        break;
-      case 'd':
-        daemon = 1;
-        break;
-      case 'l':
-        load_logfile_name = optarg;
-        break;
-      }
+    n_processed_args = grn_str_getopt(argc, argv, opts, &flags);
+
+    if (log_path) {
+      grn_default_logger_set_path(log_path);
     }
-    argc -= optind; argv += optind;
+
+    if (log_level) {
+      const char * const end = log_level + strlen(log_level);
+      const char *rest = NULL;
+      const int value = grn_atoi(log_level, end, &rest);
+      if (end != rest || value < 0 || value > 9) {
+        fprintf(stderr, "invalid log level: <%s>\n", log_level);
+        return EXIT_FAILURE;
+      }
+      grn_default_logger_set_max_level(value);
+    }
+
+    mode = (flags & RUN_MODE_MASK);
+
+    if (mode & RUN_MODE_USAGE) {
+      usage(stdout);
+      return EXIT_SUCCESS;
+    }
+
+    if ((n_processed_args < 0) ||
+        (argc - n_processed_args) != 1) {
+      usage(stderr);
+    }
+
+    db_path = argv[n_processed_args];
   }
 
   /* main */
-  if (argc != 1) {
-    usage(stderr);
-  } else {
+  {
     grn_ctx *ctx;
     msgpack_zone *mempool;
 
-    if (daemon) {
+    if (mode == RUN_MODE_DAEMON) {
       daemonize();
     }
 
     grn_init();
 
     ctx = grn_ctx_open(0);
-    if (!(grn_db_open(ctx, argv[0]))) {
+    if (!(grn_db_open(ctx, db_path))) {
       print_error("cannot open database.");
     } else {
-      if (load_logfile_name) {
+      if (log_base_path) {
         /* loading log mode */
-        load_log(ctx, load_logfile_name);
+        load_log(ctx, log_base_path);
       } else {
         /* zeromq/msgpack recv mode */
         if (!(mempool = msgpack_zone_new(MSGPACK_ZONE_CHUNK_SIZE))) {
@@ -757,7 +817,7 @@ main(int argc, char **argv)
               signal(SIGQUIT, signal_handler);
 
               zmq_setsockopt(zmq_recv_sock, ZMQ_SUBSCRIBE, "", 0);
-              thd.db_path = argv[0];
+              thd.db_path = db_path;
               thd.send_endpoint = send_endpoint;
               thd.zmq_ctx = zmq_ctx;
 
