@@ -20,10 +20,6 @@
 #include <glib/gstdio.h>
 #include <cppcutter.h>
 
-#ifdef UINT32_MAX
- #undef UINT32_MAX
-#endif  // UINT32_MAX
-
 #include <grn-assertions.h>
 #include <dat/trie.hpp>
 
@@ -51,10 +47,7 @@ namespace
     void create(GThreadFunc func, gpointer data) {
       join();
 
-      GError *glib_error = NULL;
-      thread_ = g_thread_create(func, data, TRUE, &glib_error);
-      gcut_assert_error(glib_error, cut_message("g_thread_create() failed"));
-      cppcut_assert_not_null(thread_);
+      thread_ = g_thread_new("test-trie", func, data);
     }
 
     void join() {
@@ -282,10 +275,10 @@ namespace test_dat_trie
     cppcut_assert_equal(true, trie.lcp_search("0123", 4));
     cppcut_assert_equal(false, trie.lcp_search("12345", 5));
 
-    grn::dat::UInt32 key_pos = grn::dat::UINT32_MAX;
+    grn::dat::UInt32 key_pos = grn::dat::MAX_UINT32;
 
     cppcut_assert_equal(false, trie.lcp_search("01", 2, &key_pos));
-    cppcut_assert_equal(grn::dat::UINT32_MAX, key_pos);
+    cppcut_assert_equal(grn::dat::MAX_UINT32, key_pos);
 
     cppcut_assert_equal(true, trie.lcp_search("012", 3, &key_pos));
     cppcut_assert_equal(true, trie.get_key(key_pos).is_valid());
@@ -487,7 +480,6 @@ namespace test_dat_trie
               stack.empty() ? (keyset.size() + 1) : stack.top();
           const std::pair<Keyset::iterator, bool> it_bool_pair =
               keyset.insert(std::make_pair(str, key_id));
-          const Keyset::const_iterator it = it_bool_pair.first;
           const bool to_be_inserted = it_bool_pair.second;
           if (!stack.empty() && to_be_inserted) {
             stack.pop();
@@ -622,7 +614,6 @@ namespace test_dat_trie
               stack.empty() ? (keyset.size() + 1) : stack.top();
           const std::pair<Keyset::iterator, bool> it_bool_pair =
               keyset.insert(std::make_pair(str, key_id));
-          const Keyset::const_iterator it = it_bool_pair.first;
           const bool to_be_inserted = it_bool_pair.second;
           if (!stack.empty() && to_be_inserted) {
             stack.pop();
