@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2; coding: utf-8 -*- */
 /*
-  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+  Copyright (C) 2008-2012  Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef __GRN_ASSERTIONS_H__
@@ -47,6 +47,16 @@
       __VA_ARGS__),                                                     \
     grn_test_assert_equal_id(context, expected, actual))
 
+#define grn_test_assert_equal_type(context, expected, actual, ...)      \
+  cut_trace_with_info_expression(                                       \
+    cut_test_with_user_message(                                         \
+      grn_test_assert_equal_type_helper((context),                      \
+                                        (expected), (actual),           \
+                                        #context,                       \
+                                        #expected, #actual),            \
+      __VA_ARGS__),                                                     \
+    grn_test_assert_equal_type(context, expected, actual))
+
 #define grn_test_assert_equal_record_id(context, table,                 \
                                         expected, actual, ...)          \
   cut_trace_with_info_expression(                                       \
@@ -139,17 +149,6 @@
       __VA_ARGS__),                                                     \
     grn_test_assert_equal_table(context, expected, table))
 
-#define grn_test_assert_equal_view(context, expected, view,             \
-                                   text_column_name, ...)               \
-  cut_trace_with_info_expression(                                       \
-    cut_test_with_user_message(                                         \
-      grn_test_assert_equal_view_helper((context), (expected), (view),  \
-                                        (text_column_name),             \
-                                        #expected, #view,               \
-                                        #text_column_name),             \
-      __VA_ARGS__),                                                     \
-    grn_test_assert_equal_view(context, expected, view, text_column_name))
-
 #define grn_test_assert_send_command_error(context, expected_rc,        \
                                            expected_message, command,   \
                                            ...)                         \
@@ -182,6 +181,13 @@ void     grn_test_assert_equal_id_helper(grn_ctx     *context,
                                          const gchar *expression_context,
                                          const gchar *expression_expected,
                                          const gchar *expression_actual);
+void     grn_test_assert_equal_type_helper
+                                        (grn_ctx     *context,
+                                         unsigned char expected,
+                                         unsigned char actual,
+                                         const gchar *expression_context,
+                                         const gchar *expression_expected,
+                                         const gchar *expression_actual);
 void     grn_test_assert_equal_record_id_helper
                                         (grn_ctx     *context,
                                          grn_obj     *table,
@@ -232,14 +238,6 @@ void     grn_test_assert_equal_table_helper
                                          const gchar *expected_expression,
                                          const gchar *select_result_expression,
                                          const gchar *text_column_name_expression);
-void     grn_test_assert_equal_view_helper
-                                        (grn_ctx     *context,
-                                         const GList *expected,
-                                         grn_obj     *view,
-                                         const gchar *text_column_name,
-                                         const gchar *expected_expression,
-                                         const gchar *view_expression,
-                                         const gchar *text_column_name_expression);
 void     grn_test_assert_send_command_error_helper
                                         (grn_ctx     *context,
                                          grn_rc       expected_rc,
