@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <errno.h>
@@ -195,7 +195,7 @@ grn_test_server_start(GrnTestServer *server, GError **error)
   if (!gcut_egg_hatch(priv->egg, error))
     return FALSE;
 
-  g_usleep(G_USEC_PER_SEC); /* FIXME: use select */
+  g_usleep(1.5 * G_USEC_PER_SEC); /* FIXME: use select */
 
   return TRUE;
 }
