@@ -41,12 +41,6 @@ elif test x"$CUTTER_CHECK_LEAK" = x"yes"; then
     CUTTER_ARGS="--keep-opening-modules"
 fi
 
-RUBY_TEST_ARGS=
-if test x"$PRIORITY_MODE" = x"yes"; then
-    RUBY_TEST_ARGS="$RUBY_TEST_ARGS --priority-mode"
-    # CUTTER_ARGS="$CUTTER_ARGS --priority-mode"
-fi
-
 CUTTER_ARGS="$CUTTER_ARGS -s $BASE_DIR"
 if test x"$CUTTER_VERBOSE" = x"yes"; then
     CUTTER_ARGS="$CUTTER_ARGS -v v"
@@ -79,38 +73,4 @@ for tmpfs in $tmpfs_candidates; do
     fi
 done
 
-no_test=1
-
-cutter_result=0
-if test "$NO_CUTTER" != "yes" -a -n "$CUTTER"; then
-    $CUTTER_WRAPPER $CUTTER $CUTTER_ARGS "$@" $BUILD_DIR
-    cutter_result=$?
-    no_test=0
-fi
-
-ruby_result=0
-if test "$NO_RUBY" != "yes" -a -n "$RUBY"; then
-    : ${TEST_UNIT_MAX_DIFF_TARGET_STRING_SIZE:=30000}
-    export TEST_UNIT_MAX_DIFF_TARGET_STRING_SIZE
-    BUNDLE_GEMFILE="$BUILD_DIR/Gemfile"
-    export BUNDLE_GEMFILE
-    if [ ! -e "$BUNDLE_GEMFILE" ]; then
-	ln -s "$BASE_DIR/Gemfile" "$BUNDLE_GEMFILE"
-    fi
-    if ! type bundle > /dev/null; then
-	$RUBY -S gem install bundler
-    fi
-    if [ ! -e "$BUNDLE_GEMFILE.lock" -o \
-	 "$BUNDLE_GEMFILE" -nt "$BUNDLE_GEMFILE.lock" ]; then
-	$RUBY -S bundle install
-    fi
-    $RUBY $BASE_DIR/run-test.rb $RUBY_TEST_ARGS "$@"
-    ruby_result=$?
-    no_test=0
-fi
-
-if [ $no_test = 0 -a $cutter_result = 0 -a $ruby_result = 0 ]; then
-    exit 0
-else
-    exit 1
-fi
+$CUTTER_WRAPPER $CUTTER $CUTTER_ARGS "$@" $BUILD_DIR
